"""Mermaid entity relations."""

import logging
from enum import Enum

from konstantin_docs.dia.base import BaseDiagram
from konstantin_docs.dia.base import Image
from konstantin_docs.service import kroki

from typing_extensions import Self

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler())


TEMPLATE: str = """%%{{init: {{'theme':'neutral'}}}}%%
erDiagram
{rels}
{entities}
"""

TEMPLATE_REL: str = (
    "    {entity_left} {entity_left_card}--{entity_right_card}"
    " {entity_right} : {label}\n"
)

TEMPLATE_ATTR: str = "        {type} {name} {key} {comment}"

TEMPLATE_ENTITY: str = """    {name} {{
{attr}
    }}

"""


class Attr:
    """Атрибут."""

    class Keys(Enum):
        """Типы ключей атрибутов."""

        PK = "PK"
        FK = "FK"
        NO = ""

    class Types(Enum):
        """Тип атрибута."""

        INT = "int"
        STR = "str"
        DATETIME = "datetime"
        BOOL = "bool"

    __ty: Types
    __name: str
    __key: Keys
    __comment: str

    def __init__(
        self: Self,
        name: str,
        ty: Types,
        comment: str = "",
        key: Keys = Keys.NO,
    ) -> None:
        """Атрибут.

        :param ty: тип данных
        :param name: название атрибута
        :param key: ключ
        :param comment: комментарий
        """
        self.__ty = ty
        self.__name = name
        self.__key = key
        self.__comment = comment

    def __repr__(self: Self) -> str:
        """Represend as string.

        :return: string representation
        """
        return TEMPLATE_ATTR.format(
            type=self.__ty.value,
            name=self.__name,
            key=self.__key.value,
            comment="" if not self.__comment else f'"{self.__comment}"',
        )


class Entity:
    """Entity."""

    __name: str
    __attr: list[Attr]

    def __init__(self: Self, name: str, attr: list[Attr]) -> None:
        """Entity.

        :param name: название
        :param attr: список атрибутов
        """
        self.__name = name
        self.__attr = attr

    @property
    def name(self: Self) -> str:
        """Название.

        :return: строка с названием
        """
        return self.__name

    def __repr__(self: Self) -> str:
        """Represend as string.

        :return: string representation
        """
        return TEMPLATE_ENTITY.format(
            name=self.__name,
            attr="\n".join([repr(a) for a in self.__attr]),
        )


class Relation:
    """Relation."""

    class Cardinalities(Enum):
        """Типы отношений."""

        c_0_1 = "|oo|"
        c_1 = "||||"
        c_0_inf = "}oo{"
        c_1_inf = "}||{"

    def __init__(
        self: Self,
        left: Entity,
        left_card: Cardinalities,
        right_card: Cardinalities,
        right: Entity,
        label: str = "label",
    ) -> None:
        """Relation.

        :param left: Entity слева
        :param right: Entity справа
        :param left_card: Cardinality of left entity
        :param right_card: Cardinality of right entity
        :param label: describes the relationship from the perspective
            of the first entity
        """
        self.__left = left
        self.__right = right
        self.__left_card = left_card
        self.__right_card = right_card
        self.__label = label

    @property
    def left(self: Self) -> Entity:
        """Entity слева.

        :return: entity слева
        """
        return self.__left

    @property
    def right(self: Self) -> Entity:
        """Entity справа.

        :return: entity справа
        """
        return self.__right

    def __repr__(self: Self) -> str:
        """Represend as string.

        :return: string representation
        """
        return TEMPLATE_REL.format(
            entity_left=self.__left.name,
            entity_left_card=self.__left_card.value[0:2],
            entity_right_card=self.__right_card.value[2:],
            entity_right=self.__right.name,
            label=self.__label,
        )


class Diagram(BaseDiagram):
    """Класс диаграммы."""

    __rels: list[Relation]

    def __init__(
        self: Self,
        filename: str,
        rels: list[Relation],
    ) -> None:
        """Класс диаграммы.

        :param filename: имя файла
        :param rels: список отношений между Entities
        """
        super().__init__(filename)
        self.__rels = rels

    def get_images(self: Self) -> tuple[Image]:
        """Возвращает кортеж изображений.

        :return: кортеж изображений
        """
        images: list[Image] = []
        text = repr(self)
        images.append(self._get_text_file(".puml"))
        try:
            for fmt in (kroki.OutputFormats.SVG,):
                images.append(
                    Image(
                        filename=self.filename + "." + fmt.value,
                        content=kroki.get_image(
                            source=text,
                            diagram_type=kroki.DiagramTypes.MERMAID,
                            output_format=fmt,
                        ),
                    ),
                )
        except RuntimeError as exc:
            logger.exception(exc)
        return tuple(images)

    def __repr__(self: Self) -> str:
        """Represent as string.

        :return: string representation
        """
        entities: set[Entity] = set()
        for rel in self.__rels:
            entities.add(rel.left)
            entities.add(rel.right)
        return TEMPLATE.format(
            rels="".join([repr(rel) for rel in self.__rels]),
            entities="".join([repr(e) for e in entities]),
        )



"""font awesom 5 lib."""


from konstantin_docs.dia.c4.base import BaseSprites


class FontAwesome5Lib(BaseSprites):
    """Библиотека спрайтов."""

    FA_500PX = "500px"
    ACCESSIBLE_ICON = "accessible_icon"
    ACCUSOFT = "accusoft"
    ACQUISITIONS_INCORPORATED = "acquisitions_incorporated"
    AD = "ad"
    ADDRESS_BOOK = "address_book"
    ADDRESS_CARD = "address_card"
    ADJUST = "adjust"
    ADN = "adn"
    ADVERSAL = "adversal"
    AFFILIATETHEME = "affiliatetheme"
    AIR_FRESHENER = "air_freshener"
    AIRBNB = "airbnb"
    ALGOLIA = "algolia"
    ALIGN_CENTER = "align_center"
    ALIGN_JUSTIFY = "align_justify"
    ALIGN_LEFT = "align_left"
    ALIGN_RIGHT = "align_right"
    ALIPAY = "alipay"
    ALLERGIES = "allergies"
    AMAZON = "amazon"
    AMAZON_PAY = "amazon_pay"
    AMBULANCE = "ambulance"
    AMERICAN_SIGN_LANGUAGE_INTERPRETING = "american_sign_language_interpreting"
    AMILIA = "amilia"
    ANCHOR = "anchor"
    ANDROID = "android"
    ANGELLIST = "angellist"
    ANGLE_DOUBLE_DOWN = "angle_double_down"
    ANGLE_DOUBLE_LEFT = "angle_double_left"
    ANGLE_DOUBLE_RIGHT = "angle_double_right"
    ANGLE_DOUBLE_UP = "angle_double_up"
    ANGLE_DOWN = "angle_down"
    ANGLE_LEFT = "angle_left"
    ANGLE_RIGHT = "angle_right"
    ANGLE_UP = "angle_up"
    ANGRY = "angry"
    ANGRYCREATIVE = "angrycreative"
    ANGULAR = "angular"
    ANKH = "ankh"
    APP_STORE = "app_store"
    APP_STORE_IOS = "app_store_ios"
    APPER = "apper"
    APPLE = "apple"
    APPLE_ALT = "apple_alt"
    APPLE_PAY = "apple_pay"
    ARCHIVE = "archive"
    ARCHWAY = "archway"
    ARROW_ALT_CIRCLE_DOWN = "arrow_alt_circle_down"
    ARROW_ALT_CIRCLE_LEFT = "arrow_alt_circle_left"
    ARROW_ALT_CIRCLE_RIGHT = "arrow_alt_circle_right"
    ARROW_ALT_CIRCLE_UP = "arrow_alt_circle_up"
    ARROW_CIRCLE_DOWN = "arrow_circle_down"
    ARROW_CIRCLE_LEFT = "arrow_circle_left"
    ARROW_CIRCLE_RIGHT = "arrow_circle_right"
    ARROW_CIRCLE_UP = "arrow_circle_up"
    ARROW_DOWN = "arrow_down"
    ARROW_LEFT = "arrow_left"
    ARROW_RIGHT = "arrow_right"
    ARROW_UP = "arrow_up"
    ARROWS_ALT = "arrows_alt"
    ARROWS_ALT_H = "arrows_alt_h"
    ARROWS_ALT_V = "arrows_alt_v"
    ARTSTATION = "artstation"
    ASSISTIVE_LISTENING_SYSTEMS = "assistive_listening_systems"
    ASTERISK = "asterisk"
    ASYMMETRIK = "asymmetrik"
    AT = "at"
    ATLAS = "atlas"
    ATLASSIAN = "atlassian"
    ATOM = "atom"
    AUDIBLE = "audible"
    AUDIO_DESCRIPTION = "audio_description"
    AUTOPREFIXER = "autoprefixer"
    AVIANEX = "avianex"
    AVIATO = "aviato"
    AWARD = "award"
    AWS = "aws"
    BABY = "baby"
    BABY_CARRIAGE = "baby_carriage"
    BACKSPACE = "backspace"
    BACKWARD = "backward"
    BACON = "bacon"
    BACTERIA = "bacteria"
    BACTERIUM = "bacterium"
    BAHAI = "bahai"
    BALANCE_SCALE = "balance_scale"
    BALANCE_SCALE_LEFT = "balance_scale_left"
    BALANCE_SCALE_RIGHT = "balance_scale_right"
    BAN = "ban"
    BAND_AID = "band_aid"
    BANDCAMP = "bandcamp"
    BARCODE = "barcode"
    BARS = "bars"
    BASEBALL_BALL = "baseball_ball"
    BASKETBALL_BALL = "basketball_ball"
    BATH = "bath"
    BATTERY_EMPTY = "battery_empty"
    BATTERY_FULL = "battery_full"
    BATTERY_HALF = "battery_half"
    BATTERY_QUARTER = "battery_quarter"
    BATTERY_THREE_QUARTERS = "battery_three_quarters"
    BATTLE_NET = "battle_net"
    BED = "bed"
    BEER = "beer"
    BEHANCE = "behance"
    BEHANCE_SQUARE = "behance_square"
    BELL = "bell"
    BELL_SLASH = "bell_slash"
    BEZIER_CURVE = "bezier_curve"
    BIBLE = "bible"
    BICYCLE = "bicycle"
    BIKING = "biking"
    BIMOBJECT = "bimobject"
    BINOCULARS = "binoculars"
    BIOHAZARD = "biohazard"
    BIRTHDAY_CAKE = "birthday_cake"
    BITBUCKET = "bitbucket"
    BITCOIN = "bitcoin"
    BITY = "bity"
    BLACK_TIE = "black_tie"
    BLACKBERRY = "blackberry"
    BLENDER = "blender"
    BLENDER_PHONE = "blender_phone"
    BLIND = "blind"
    BLOG = "blog"
    BLOGGER = "blogger"
    BLOGGER_B = "blogger_b"
    BLUETOOTH = "bluetooth"
    BLUETOOTH_B = "bluetooth_b"
    BOLD = "bold"
    BOLT = "bolt"
    BOMB = "bomb"
    BONE = "bone"
    BONG = "bong"
    BOOK = "book"
    BOOK_DEAD = "book_dead"
    BOOK_MEDICAL = "book_medical"
    BOOK_OPEN = "book_open"
    BOOK_READER = "book_reader"
    BOOKMARK = "bookmark"
    BOOTSTRAP = "bootstrap"
    BORDER_ALL = "border_all"
    BORDER_NONE = "border_none"
    BORDER_STYLE = "border_style"
    BOWLING_BALL = "bowling_ball"
    BOX = "box"
    BOX_OPEN = "box_open"
    BOX_TISSUE = "box_tissue"
    BOXES = "boxes"
    BRAILLE = "braille"
    BRAIN = "brain"
    BREAD_SLICE = "bread_slice"
    BRIEFCASE = "briefcase"
    BRIEFCASE_MEDICAL = "briefcase_medical"
    BROADCAST_TOWER = "broadcast_tower"
    BROOM = "broom"
    BRUSH = "brush"
    BTC = "btc"
    BUFFER = "buffer"
    BUG = "bug"
    BUILDING = "building"
    BULLHORN = "bullhorn"
    BULLSEYE = "bullseye"
    BURN = "burn"
    BUROMOBELEXPERTE = "buromobelexperte"
    BUS = "bus"
    BUS_ALT = "bus_alt"
    BUSINESS_TIME = "business_time"
    BUY_N_LARGE = "buy_n_large"
    BUYSELLADS = "buysellads"
    CALCULATOR = "calculator"
    CALENDAR = "calendar"
    CALENDAR_ALT = "calendar_alt"
    CALENDAR_CHECK = "calendar_check"
    CALENDAR_DAY = "calendar_day"
    CALENDAR_MINUS = "calendar_minus"
    CALENDAR_PLUS = "calendar_plus"
    CALENDAR_TIMES = "calendar_times"
    CALENDAR_WEEK = "calendar_week"
    CAMERA = "camera"
    CAMERA_RETRO = "camera_retro"
    CAMPGROUND = "campground"
    CANADIAN_MAPLE_LEAF = "canadian_maple_leaf"
    CANDY_CANE = "candy_cane"
    CANNABIS = "cannabis"
    CAPSULES = "capsules"
    CAR = "car"
    CAR_ALT = "car_alt"
    CAR_BATTERY = "car_battery"
    CAR_CRASH = "car_crash"
    CAR_SIDE = "car_side"
    CARAVAN = "caravan"
    CARET_DOWN = "caret_down"
    CARET_LEFT = "caret_left"
    CARET_RIGHT = "caret_right"
    CARET_SQUARE_DOWN = "caret_square_down"
    CARET_SQUARE_LEFT = "caret_square_left"
    CARET_SQUARE_RIGHT = "caret_square_right"
    CARET_SQUARE_UP = "caret_square_up"
    CARET_UP = "caret_up"
    CARROT = "carrot"
    CART_ARROW_DOWN = "cart_arrow_down"
    CART_PLUS = "cart_plus"
    CASH_REGISTER = "cash_register"
    CAT = "cat"
    CC_AMAZON_PAY = "cc_amazon_pay"
    CC_AMEX = "cc_amex"
    CC_APPLE_PAY = "cc_apple_pay"
    CC_DINERS_CLUB = "cc_diners_club"
    CC_DISCOVER = "cc_discover"
    CC_JCB = "cc_jcb"
    CC_MASTERCARD = "cc_mastercard"
    CC_PAYPAL = "cc_paypal"
    CC_STRIPE = "cc_stripe"
    CC_VISA = "cc_visa"
    CENTERCODE = "centercode"
    CENTOS = "centos"
    CERTIFICATE = "certificate"
    CHAIR = "chair"
    CHALKBOARD = "chalkboard"
    CHALKBOARD_TEACHER = "chalkboard_teacher"
    CHARGING_STATION = "charging_station"
    CHART_AREA = "chart_area"
    CHART_BAR = "chart_bar"
    CHART_LINE = "chart_line"
    CHART_PIE = "chart_pie"
    CHECK = "check"
    CHECK_CIRCLE = "check_circle"
    CHECK_DOUBLE = "check_double"
    CHECK_SQUARE = "check_square"
    CHEESE = "cheese"
    CHESS = "chess"
    CHESS_BISHOP = "chess_bishop"
    CHESS_BOARD = "chess_board"
    CHESS_KING = "chess_king"
    CHESS_KNIGHT = "chess_knight"
    CHESS_PAWN = "chess_pawn"
    CHESS_QUEEN = "chess_queen"
    CHESS_ROOK = "chess_rook"
    CHEVRON_CIRCLE_DOWN = "chevron_circle_down"
    CHEVRON_CIRCLE_LEFT = "chevron_circle_left"
    CHEVRON_CIRCLE_RIGHT = "chevron_circle_right"
    CHEVRON_CIRCLE_UP = "chevron_circle_up"
    CHEVRON_DOWN = "chevron_down"
    CHEVRON_LEFT = "chevron_left"
    CHEVRON_RIGHT = "chevron_right"
    CHEVRON_UP = "chevron_up"
    CHILD = "child"
    CHROME = "chrome"
    CHROMECAST = "chromecast"
    CHURCH = "church"
    CIRCLE = "circle"
    CIRCLE_NOTCH = "circle_notch"
    CITY = "city"
    CLINIC_MEDICAL = "clinic_medical"
    CLIPBOARD = "clipboard"
    CLIPBOARD_CHECK = "clipboard_check"
    CLIPBOARD_LIST = "clipboard_list"
    CLOCK = "clock"
    CLONE = "clone"
    CLOSED_CAPTIONING = "closed_captioning"
    CLOUD = "cloud"
    CLOUD_DOWNLOAD_ALT = "cloud_download_alt"
    CLOUD_MEATBALL = "cloud_meatball"
    CLOUD_MOON = "cloud_moon"
    CLOUD_MOON_RAIN = "cloud_moon_rain"
    CLOUD_RAIN = "cloud_rain"
    CLOUD_SHOWERS_HEAVY = "cloud_showers_heavy"
    CLOUD_SUN = "cloud_sun"
    CLOUD_SUN_RAIN = "cloud_sun_rain"
    CLOUD_UPLOAD_ALT = "cloud_upload_alt"
    CLOUDFLARE = "cloudflare"
    CLOUDSCALE = "cloudscale"
    CLOUDSMITH = "cloudsmith"
    CLOUDVERSIFY = "cloudversify"
    COCKTAIL = "cocktail"
    CODE = "code"
    CODE_BRANCH = "code_branch"
    CODEPEN = "codepen"
    CODIEPIE = "codiepie"
    COFFEE = "coffee"
    COG = "cog"
    COGS = "cogs"
    COINS = "coins"
    COLUMNS = "columns"
    COMMENT = "comment"
    COMMENT_ALT = "comment_alt"
    COMMENT_DOLLAR = "comment_dollar"
    COMMENT_DOTS = "comment_dots"
    COMMENT_MEDICAL = "comment_medical"
    COMMENT_SLASH = "comment_slash"
    COMMENTS = "comments"
    COMMENTS_DOLLAR = "comments_dollar"
    COMPACT_DISC = "compact_disc"
    COMPASS = "compass"
    COMPRESS = "compress"
    COMPRESS_ALT = "compress_alt"
    COMPRESS_ARROWS_ALT = "compress_arrows_alt"
    CONCIERGE_BELL = "concierge_bell"
    CONFLUENCE = "confluence"
    CONNECTDEVELOP = "connectdevelop"
    CONTAO = "contao"
    COOKIE = "cookie"
    COOKIE_BITE = "cookie_bite"
    COPY = "copy"
    COPYRIGHT = "copyright"
    COTTON_BUREAU = "cotton_bureau"
    COUCH = "couch"
    CPANEL = "cpanel"
    CREATIVE_COMMONS = "creative_commons"
    CREATIVE_COMMONS_BY = "creative_commons_by"
    CREATIVE_COMMONS_NC = "creative_commons_nc"
    CREATIVE_COMMONS_NC_EU = "creative_commons_nc_eu"
    CREATIVE_COMMONS_NC_JP = "creative_commons_nc_jp"
    CREATIVE_COMMONS_ND = "creative_commons_nd"
    CREATIVE_COMMONS_PD = "creative_commons_pd"
    CREATIVE_COMMONS_PD_ALT = "creative_commons_pd_alt"
    CREATIVE_COMMONS_REMIX = "creative_commons_remix"
    CREATIVE_COMMONS_SA = "creative_commons_sa"
    CREATIVE_COMMONS_SAMPLING = "creative_commons_sampling"
    CREATIVE_COMMONS_SAMPLING_PLUS = "creative_commons_sampling_plus"
    CREATIVE_COMMONS_SHARE = "creative_commons_share"
    CREATIVE_COMMONS_ZERO = "creative_commons_zero"
    CREDIT_CARD = "credit_card"
    CRITICAL_ROLE = "critical_role"
    CROP = "crop"
    CROP_ALT = "crop_alt"
    CROSS = "cross"
    CROSSHAIRS = "crosshairs"
    CROW = "crow"
    CROWN = "crown"
    CRUTCH = "crutch"
    CSS3 = "css3"
    CSS3_ALT = "css3_alt"
    CUBE = "cube"
    CUBES = "cubes"
    CUT = "cut"
    CUTTLEFISH = "cuttlefish"
    D_AND_D = "d_and_d"
    D_AND_D_BEYOND = "d_and_d_beyond"
    DAILYMOTION = "dailymotion"
    DASHCUBE = "dashcube"
    DATABASE = "database"
    DEAF = "deaf"
    DEEZER = "deezer"
    DELICIOUS = "delicious"
    DEMOCRAT = "democrat"
    DEPLOYDOG = "deploydog"
    DESKPRO = "deskpro"
    DESKTOP = "desktop"
    DEV = "dev"
    DEVIANTART = "deviantart"
    DHARMACHAKRA = "dharmachakra"
    DHL = "dhl"
    DIAGNOSES = "diagnoses"
    DIASPORA = "diaspora"
    DICE = "dice"
    DICE_D20 = "dice_d20"
    DICE_D6 = "dice_d6"
    DICE_FIVE = "dice_five"
    DICE_FOUR = "dice_four"
    DICE_ONE = "dice_one"
    DICE_SIX = "dice_six"
    DICE_THREE = "dice_three"
    DICE_TWO = "dice_two"
    DIGG = "digg"
    DIGITAL_OCEAN = "digital_ocean"
    DIGITAL_TACHOGRAPH = "digital_tachograph"
    DIRECTIONS = "directions"
    DISCORD = "discord"
    DISCOURSE = "discourse"
    DISEASE = "disease"
    DIVIDE = "divide"
    DIZZY = "dizzy"
    DNA = "dna"
    DOCHUB = "dochub"
    DOCKER = "docker"
    DOG = "dog"
    DOLLAR_SIGN = "dollar_sign"
    DOLLY = "dolly"
    DOLLY_FLATBED = "dolly_flatbed"
    DONATE = "donate"
    DOOR_CLOSED = "door_closed"
    DOOR_OPEN = "door_open"
    DOT_CIRCLE = "dot_circle"
    DOVE = "dove"
    DOWNLOAD = "download"
    DRAFT2DIGITAL = "draft2digital"
    DRAFTING_COMPASS = "drafting_compass"
    DRAGON = "dragon"
    DRAW_POLYGON = "draw_polygon"
    DRIBBBLE = "dribbble"
    DRIBBBLE_SQUARE = "dribbble_square"
    DROPBOX = "dropbox"
    DRUM = "drum"
    DRUM_STEELPAN = "drum_steelpan"
    DRUMSTICK_BITE = "drumstick_bite"
    DRUPAL = "drupal"
    DUMBBELL = "dumbbell"
    DUMPSTER = "dumpster"
    DUMPSTER_FIRE = "dumpster_fire"
    DUNGEON = "dungeon"
    DYALOG = "dyalog"
    EARLYBIRDS = "earlybirds"
    EBAY = "ebay"
    EDGE = "edge"
    EDGE_LEGACY = "edge_legacy"
    EDIT = "edit"
    EGG = "egg"
    EJECT = "eject"
    ELEMENTOR = "elementor"
    ELLIPSIS_H = "ellipsis_h"
    ELLIPSIS_V = "ellipsis_v"
    ELLO = "ello"
    EMBER = "ember"
    EMPIRE = "empire"
    ENVELOPE = "envelope"
    ENVELOPE_OPEN = "envelope_open"
    ENVELOPE_OPEN_TEXT = "envelope_open_text"
    ENVELOPE_SQUARE = "envelope_square"
    ENVIRA = "envira"
    EQUALS = "equals"
    ERASER = "eraser"
    ERLANG = "erlang"
    ETHEREUM = "ethereum"
    ETHERNET = "ethernet"
    ETSY = "etsy"
    EURO_SIGN = "euro_sign"
    EVERNOTE = "evernote"
    EXCHANGE_ALT = "exchange_alt"
    EXCLAMATION = "exclamation"
    EXCLAMATION_CIRCLE = "exclamation_circle"
    EXCLAMATION_TRIANGLE = "exclamation_triangle"
    EXPAND = "expand"
    EXPAND_ALT = "expand_alt"
    EXPAND_ARROWS_ALT = "expand_arrows_alt"
    EXPEDITEDSSL = "expeditedssl"
    EXTERNAL_LINK_ALT = "external_link_alt"
    EXTERNAL_LINK_SQUARE_ALT = "external_link_square_alt"
    EYE = "eye"
    EYE_DROPPER = "eye_dropper"
    EYE_SLASH = "eye_slash"
    FACEBOOK = "facebook"
    FACEBOOK_F = "facebook_f"
    FACEBOOK_MESSENGER = "facebook_messenger"
    FACEBOOK_SQUARE = "facebook_square"
    FAN = "fan"
    FANTASY_FLIGHT_GAMES = "fantasy_flight_games"
    FAST_BACKWARD = "fast_backward"
    FAST_FORWARD = "fast_forward"
    FAUCET = "faucet"
    FAX = "fax"
    FEATHER = "feather"
    FEATHER_ALT = "feather_alt"
    FEDEX = "fedex"
    FEDORA = "fedora"
    FEMALE = "female"
    FIGHTER_JET = "fighter_jet"
    FIGMA = "figma"
    FILE = "file"
    FILE_ALT = "file_alt"
    FILE_ARCHIVE = "file_archive"
    FILE_AUDIO = "file_audio"
    FILE_CODE = "file_code"
    FILE_CONTRACT = "file_contract"
    FILE_CSV = "file_csv"
    FILE_DOWNLOAD = "file_download"
    FILE_EXCEL = "file_excel"
    FILE_EXPORT = "file_export"
    FILE_IMAGE = "file_image"
    FILE_IMPORT = "file_import"
    FILE_INVOICE = "file_invoice"
    FILE_INVOICE_DOLLAR = "file_invoice_dollar"
    FILE_MEDICAL = "file_medical"
    FILE_MEDICAL_ALT = "file_medical_alt"
    FILE_PDF = "file_pdf"
    FILE_POWERPOINT = "file_powerpoint"
    FILE_PRESCRIPTION = "file_prescription"
    FILE_SIGNATURE = "file_signature"
    FILE_UPLOAD = "file_upload"
    FILE_VIDEO = "file_video"
    FILE_WORD = "file_word"
    FILL = "fill"
    FILL_DRIP = "fill_drip"
    FILM = "film"
    FILTER = "filter"
    FINGERPRINT = "fingerprint"
    FIRE = "fire"
    FIRE_ALT = "fire_alt"
    FIRE_EXTINGUISHER = "fire_extinguisher"
    FIREFOX = "firefox"
    FIREFOX_BROWSER = "firefox_browser"
    FIRST_AID = "first_aid"
    FIRST_ORDER = "first_order"
    FIRST_ORDER_ALT = "first_order_alt"
    FIRSTDRAFT = "firstdraft"
    FISH = "fish"
    FIST_RAISED = "fist_raised"
    FLAG = "flag"
    FLAG_CHECKERED = "flag_checkered"
    FLAG_USA = "flag_usa"
    FLASK = "flask"
    FLICKR = "flickr"
    FLIPBOARD = "flipboard"
    FLUSHED = "flushed"
    FLY = "fly"
    FOLDER = "folder"
    FOLDER_MINUS = "folder_minus"
    FOLDER_OPEN = "folder_open"
    FOLDER_PLUS = "folder_plus"
    FONT = "font"
    FONT_AWESOME = "font_awesome"
    FONT_AWESOME_ALT = "font_awesome_alt"
    FONT_AWESOME_FLAG = "font_awesome_flag"
    FONTICONS = "fonticons"
    FONTICONS_FI = "fonticons_fi"
    FOOTBALL_BALL = "football_ball"
    FORT_AWESOME = "fort_awesome"
    FORT_AWESOME_ALT = "fort_awesome_alt"
    FORUMBEE = "forumbee"
    FORWARD = "forward"
    FOURSQUARE = "foursquare"
    FREE_CODE_CAMP = "free_code_camp"
    FREEBSD = "freebsd"
    FROG = "frog"
    FROWN = "frown"
    FROWN_OPEN = "frown_open"
    FULCRUM = "fulcrum"
    FUNNEL_DOLLAR = "funnel_dollar"
    FUTBOL = "futbol"
    GALACTIC_REPUBLIC = "galactic_republic"
    GALACTIC_SENATE = "galactic_senate"
    GAMEPAD = "gamepad"
    GAS_PUMP = "gas_pump"
    GAVEL = "gavel"
    GEM = "gem"
    GENDERLESS = "genderless"
    GET_POCKET = "get_pocket"
    GG = "gg"
    GG_CIRCLE = "gg_circle"
    GHOST = "ghost"
    GIFT = "gift"
    GIFTS = "gifts"
    GIT = "git"
    GIT_ALT = "git_alt"
    GIT_SQUARE = "git_square"
    GITHUB = "github"
    GITHUB_ALT = "github_alt"
    GITHUB_SQUARE = "github_square"
    GITKRAKEN = "gitkraken"
    GITLAB = "gitlab"
    GITTER = "gitter"
    GLASS_CHEERS = "glass_cheers"
    GLASS_MARTINI = "glass_martini"
    GLASS_MARTINI_ALT = "glass_martini_alt"
    GLASS_WHISKEY = "glass_whiskey"
    GLASSES = "glasses"
    GLIDE = "glide"
    GLIDE_G = "glide_g"
    GLOBE = "globe"
    GLOBE_AFRICA = "globe_africa"
    GLOBE_AMERICAS = "globe_americas"
    GLOBE_ASIA = "globe_asia"
    GLOBE_EUROPE = "globe_europe"
    GOFORE = "gofore"
    GOLF_BALL = "golf_ball"
    GOODREADS = "goodreads"
    GOODREADS_G = "goodreads_g"
    GOOGLE = "google"
    GOOGLE_DRIVE = "google_drive"
    GOOGLE_PAY = "google_pay"
    GOOGLE_PLAY = "google_play"
    GOOGLE_PLUS = "google_plus"
    GOOGLE_PLUS_G = "google_plus_g"
    GOOGLE_PLUS_SQUARE = "google_plus_square"
    GOOGLE_WALLET = "google_wallet"
    GOPURAM = "gopuram"
    GRADUATION_CAP = "graduation_cap"
    GRATIPAY = "gratipay"
    GRAV = "grav"
    GREATER_THAN = "greater_than"
    GREATER_THAN_EQUAL = "greater_than_equal"
    GRIMACE = "grimace"
    GRIN = "grin"
    GRIN_ALT = "grin_alt"
    GRIN_BEAM = "grin_beam"
    GRIN_BEAM_SWEAT = "grin_beam_sweat"
    GRIN_HEARTS = "grin_hearts"
    GRIN_SQUINT = "grin_squint"
    GRIN_SQUINT_TEARS = "grin_squint_tears"
    GRIN_STARS = "grin_stars"
    GRIN_TEARS = "grin_tears"
    GRIN_TONGUE = "grin_tongue"
    GRIN_TONGUE_SQUINT = "grin_tongue_squint"
    GRIN_TONGUE_WINK = "grin_tongue_wink"
    GRIN_WINK = "grin_wink"
    GRIP_HORIZONTAL = "grip_horizontal"
    GRIP_LINES = "grip_lines"
    GRIP_LINES_VERTICAL = "grip_lines_vertical"
    GRIP_VERTICAL = "grip_vertical"
    GRIPFIRE = "gripfire"
    GRUNT = "grunt"
    GUILDED = "guilded"
    GUITAR = "guitar"
    GULP = "gulp"
    H_SQUARE = "h_square"
    HACKER_NEWS = "hacker_news"
    HACKER_NEWS_SQUARE = "hacker_news_square"
    HACKERRANK = "hackerrank"
    HAMBURGER = "hamburger"
    HAMMER = "hammer"
    HAMSA = "hamsa"
    HAND_HOLDING = "hand_holding"
    HAND_HOLDING_HEART = "hand_holding_heart"
    HAND_HOLDING_MEDICAL = "hand_holding_medical"
    HAND_HOLDING_USD = "hand_holding_usd"
    HAND_HOLDING_WATER = "hand_holding_water"
    HAND_LIZARD = "hand_lizard"
    HAND_MIDDLE_FINGER = "hand_middle_finger"
    HAND_PAPER = "hand_paper"
    HAND_PEACE = "hand_peace"
    HAND_POINT_DOWN = "hand_point_down"
    HAND_POINT_LEFT = "hand_point_left"
    HAND_POINT_RIGHT = "hand_point_right"
    HAND_POINT_UP = "hand_point_up"
    HAND_POINTER = "hand_pointer"
    HAND_ROCK = "hand_rock"
    HAND_SCISSORS = "hand_scissors"
    HAND_SPARKLES = "hand_sparkles"
    HAND_SPOCK = "hand_spock"
    HANDS = "hands"
    HANDS_HELPING = "hands_helping"
    HANDS_WASH = "hands_wash"
    HANDSHAKE = "handshake"
    HANDSHAKE_ALT_SLASH = "handshake_alt_slash"
    HANDSHAKE_SLASH = "handshake_slash"
    HANUKIAH = "hanukiah"
    HARD_HAT = "hard_hat"
    HASHTAG = "hashtag"
    HAT_COWBOY = "hat_cowboy"
    HAT_COWBOY_SIDE = "hat_cowboy_side"
    HAT_WIZARD = "hat_wizard"
    HDD = "hdd"
    HEAD_SIDE_COUGH = "head_side_cough"
    HEAD_SIDE_COUGH_SLASH = "head_side_cough_slash"
    HEAD_SIDE_MASK = "head_side_mask"
    HEAD_SIDE_VIRUS = "head_side_virus"
    HEADING = "heading"
    HEADPHONES = "headphones"
    HEADPHONES_ALT = "headphones_alt"
    HEADSET = "headset"
    HEART = "heart"
    HEART_BROKEN = "heart_broken"
    HEARTBEAT = "heartbeat"
    HELICOPTER = "helicopter"
    HIGHLIGHTER = "highlighter"
    HIKING = "hiking"
    HIPPO = "hippo"
    HIPS = "hips"
    HIRE_A_HELPER = "hire_a_helper"
    HISTORY = "history"
    HIVE = "hive"
    HOCKEY_PUCK = "hockey_puck"
    HOLLY_BERRY = "holly_berry"
    HOME = "home"
    HOOLI = "hooli"
    HORNBILL = "hornbill"
    HORSE = "horse"
    HORSE_HEAD = "horse_head"
    HOSPITAL = "hospital"
    HOSPITAL_ALT = "hospital_alt"
    HOSPITAL_SYMBOL = "hospital_symbol"
    HOSPITAL_USER = "hospital_user"
    HOT_TUB = "hot_tub"
    HOTDOG = "hotdog"
    HOTEL = "hotel"
    HOTJAR = "hotjar"
    HOURGLASS = "hourglass"
    HOURGLASS_END = "hourglass_end"
    HOURGLASS_HALF = "hourglass_half"
    HOURGLASS_START = "hourglass_start"
    HOUSE_DAMAGE = "house_damage"
    HOUSE_USER = "house_user"
    HOUZZ = "houzz"
    HRYVNIA = "hryvnia"
    HTML5 = "html5"
    HUBSPOT = "hubspot"
    I_CURSOR = "i_cursor"
    ICE_CREAM = "ice_cream"
    ICICLES = "icicles"
    ICONS = "icons"
    ID_BADGE = "id_badge"
    ID_CARD = "id_card"
    ID_CARD_ALT = "id_card_alt"
    IDEAL = "ideal"
    IGLOO = "igloo"
    IMAGE = "image"
    IMAGES = "images"
    IMDB = "imdb"
    INBOX = "inbox"
    INDENT = "indent"
    INDEX = "index"
    INDUSTRY = "industry"
    INFINITY = "infinity"
    INFO = "info"
    INFO_CIRCLE = "info_circle"
    INNOSOFT = "innosoft"
    INSTAGRAM = "instagram"
    INSTAGRAM_SQUARE = "instagram_square"
    INSTALOD = "instalod"
    INTERCOM = "intercom"
    INTERNET_EXPLORER = "internet_explorer"
    INVISION = "invision"
    IOXHOST = "ioxhost"
    ITALIC = "italic"
    ITCH_IO = "itch_io"
    ITUNES = "itunes"
    ITUNES_NOTE = "itunes_note"
    JAVA = "java"
    JEDI = "jedi"
    JEDI_ORDER = "jedi_order"
    JENKINS = "jenkins"
    JIRA = "jira"
    JOGET = "joget"
    JOINT = "joint"
    JOOMLA = "joomla"
    JOURNAL_WHILLS = "journal_whills"
    JS = "js"
    JS_SQUARE = "js_square"
    JSFIDDLE = "jsfiddle"
    KAABA = "kaaba"
    KAGGLE = "kaggle"
    KEY = "key"
    KEYBASE = "keybase"
    KEYBOARD = "keyboard"
    KEYCDN = "keycdn"
    KHANDA = "khanda"
    KICKSTARTER = "kickstarter"
    KICKSTARTER_K = "kickstarter_k"
    KISS = "kiss"
    KISS_BEAM = "kiss_beam"
    KISS_WINK_HEART = "kiss_wink_heart"
    KIWI_BIRD = "kiwi_bird"
    KORVUE = "korvue"
    LANDMARK = "landmark"
    LANGUAGE = "language"
    LAPTOP = "laptop"
    LAPTOP_CODE = "laptop_code"
    LAPTOP_HOUSE = "laptop_house"
    LAPTOP_MEDICAL = "laptop_medical"
    LARAVEL = "laravel"
    LASTFM = "lastfm"
    LASTFM_SQUARE = "lastfm_square"
    LAUGH = "laugh"
    LAUGH_BEAM = "laugh_beam"
    LAUGH_SQUINT = "laugh_squint"
    LAUGH_WINK = "laugh_wink"
    LAYER_GROUP = "layer_group"
    LEAF = "leaf"
    LEANPUB = "leanpub"
    LEMON = "lemon"
    LESS = "less"
    LESS_THAN = "less_than"
    LESS_THAN_EQUAL = "less_than_equal"
    LEVEL_DOWN_ALT = "level_down_alt"
    LEVEL_UP_ALT = "level_up_alt"
    LIFE_RING = "life_ring"
    LIGHTBULB = "lightbulb"
    LINE = "line"
    LINK = "link"
    LINKEDIN = "linkedin"
    LINKEDIN_IN = "linkedin_in"
    LINODE = "linode"
    LINUX = "linux"
    LIRA_SIGN = "lira_sign"
    LIST = "list"
    LIST_ALT = "list_alt"
    LIST_OL = "list_ol"
    LIST_UL = "list_ul"
    LOCATION_ARROW = "location_arrow"
    LOCK = "lock"
    LOCK_OPEN = "lock_open"
    LONG_ARROW_ALT_DOWN = "long_arrow_alt_down"
    LONG_ARROW_ALT_LEFT = "long_arrow_alt_left"
    LONG_ARROW_ALT_RIGHT = "long_arrow_alt_right"
    LONG_ARROW_ALT_UP = "long_arrow_alt_up"
    LOW_VISION = "low_vision"
    LUGGAGE_CART = "luggage_cart"
    LUNGS = "lungs"
    LUNGS_VIRUS = "lungs_virus"
    LYFT = "lyft"
    MAGENTO = "magento"
    MAGIC = "magic"
    MAGNET = "magnet"
    MAIL_BULK = "mail_bulk"
    MAILCHIMP = "mailchimp"
    MALE = "male"
    MANDALORIAN = "mandalorian"
    MAP = "map"
    MAP_MARKED = "map_marked"
    MAP_MARKED_ALT = "map_marked_alt"
    MAP_MARKER = "map_marker"
    MAP_MARKER_ALT = "map_marker_alt"
    MAP_PIN = "map_pin"
    MAP_SIGNS = "map_signs"
    MARKDOWN = "markdown"
    MARKER = "marker"
    MARS = "mars"
    MARS_DOUBLE = "mars_double"
    MARS_STROKE = "mars_stroke"
    MARS_STROKE_H = "mars_stroke_h"
    MARS_STROKE_V = "mars_stroke_v"
    MASK = "mask"
    MASTODON = "mastodon"
    MAXCDN = "maxcdn"
    MDB = "mdb"
    MEDAL = "medal"
    MEDAPPS = "medapps"
    MEDIUM = "medium"
    MEDIUM_M = "medium_m"
    MEDKIT = "medkit"
    MEDRT = "medrt"
    MEETUP = "meetup"
    MEGAPORT = "megaport"
    MEH = "meh"
    MEH_BLANK = "meh_blank"
    MEH_ROLLING_EYES = "meh_rolling_eyes"
    MEMORY = "memory"
    MENDELEY = "mendeley"
    MENORAH = "menorah"
    MERCURY = "mercury"
    METEOR = "meteor"
    MICROBLOG = "microblog"
    MICROCHIP = "microchip"
    MICROPHONE = "microphone"
    MICROPHONE_ALT = "microphone_alt"
    MICROPHONE_ALT_SLASH = "microphone_alt_slash"
    MICROPHONE_SLASH = "microphone_slash"
    MICROSCOPE = "microscope"
    MICROSOFT = "microsoft"
    MINUS = "minus"
    MINUS_CIRCLE = "minus_circle"
    MINUS_SQUARE = "minus_square"
    MITTEN = "mitten"
    MIX = "mix"
    MIXCLOUD = "mixcloud"
    MIXER = "mixer"
    MIZUNI = "mizuni"
    MOBILE = "mobile"
    MOBILE_ALT = "mobile_alt"
    MODX = "modx"
    MONERO = "monero"
    MONEY_BILL = "money_bill"
    MONEY_BILL_ALT = "money_bill_alt"
    MONEY_BILL_WAVE = "money_bill_wave"
    MONEY_BILL_WAVE_ALT = "money_bill_wave_alt"
    MONEY_CHECK = "money_check"
    MONEY_CHECK_ALT = "money_check_alt"
    MONUMENT = "monument"
    MOON = "moon"
    MORTAR_PESTLE = "mortar_pestle"
    MOSQUE = "mosque"
    MOTORCYCLE = "motorcycle"
    MOUNTAIN = "mountain"
    MOUSE = "mouse"
    MOUSE_POINTER = "mouse_pointer"
    MUG_HOT = "mug_hot"
    MUSIC = "music"
    NAPSTER = "napster"
    NEOS = "neos"
    NETWORK_WIRED = "network_wired"
    NEUTER = "neuter"
    NEWSPAPER = "newspaper"
    NIMBLR = "nimblr"
    NODE = "node"
    NODE_JS = "node_js"
    NOT_EQUAL = "not_equal"
    NOTES_MEDICAL = "notes_medical"
    NPM = "npm"
    NS8 = "ns8"
    NUTRITIONIX = "nutritionix"
    OBJECT_GROUP = "object_group"
    OBJECT_UNGROUP = "object_ungroup"
    OCTOPUS_DEPLOY = "octopus_deploy"
    ODNOKLASSNIKI = "odnoklassniki"
    ODNOKLASSNIKI_SQUARE = "odnoklassniki_square"
    OIL_CAN = "oil_can"
    OLD_REPUBLIC = "old_republic"
    OM = "om"
    OPENCART = "opencart"
    OPENID = "openid"
    OPERA = "opera"
    OPTIN_MONSTER = "optin_monster"
    ORCID = "orcid"
    OSI = "osi"
    OTTER = "otter"
    OUTDENT = "outdent"
    PAGE4 = "page4"
    PAGELINES = "pagelines"
    PAGER = "pager"
    PAINT_BRUSH = "paint_brush"
    PAINT_ROLLER = "paint_roller"
    PALETTE = "palette"
    PALFED = "palfed"
    PALLET = "pallet"
    PAPER_PLANE = "paper_plane"
    PAPERCLIP = "paperclip"
    PARACHUTE_BOX = "parachute_box"
    PARAGRAPH = "paragraph"
    PARKING = "parking"
    PASSPORT = "passport"
    PASTAFARIANISM = "pastafarianism"
    PASTE = "paste"
    PATREON = "patreon"
    PAUSE = "pause"
    PAUSE_CIRCLE = "pause_circle"
    PAW = "paw"
    PAYPAL = "paypal"
    PEACE = "peace"
    PEN = "pen"
    PEN_ALT = "pen_alt"
    PEN_FANCY = "pen_fancy"
    PEN_NIB = "pen_nib"
    PEN_SQUARE = "pen_square"
    PENCIL_ALT = "pencil_alt"
    PENCIL_RULER = "pencil_ruler"
    PENNY_ARCADE = "penny_arcade"
    PEOPLE_ARROWS = "people_arrows"
    PEOPLE_CARRY = "people_carry"
    PEPPER_HOT = "pepper_hot"
    PERBYTE = "perbyte"
    PERCENT = "percent"
    PERCENTAGE = "percentage"
    PERISCOPE = "periscope"
    PERSON_BOOTH = "person_booth"
    PHABRICATOR = "phabricator"
    PHOENIX_FRAMEWORK = "phoenix_framework"
    PHOENIX_SQUADRON = "phoenix_squadron"
    PHONE = "phone"
    PHONE_ALT = "phone_alt"
    PHONE_SLASH = "phone_slash"
    PHONE_SQUARE = "phone_square"
    PHONE_SQUARE_ALT = "phone_square_alt"
    PHONE_VOLUME = "phone_volume"
    PHOTO_VIDEO = "photo_video"
    PHP = "php"
    PIED_PIPER = "pied_piper"
    PIED_PIPER_ALT = "pied_piper_alt"
    PIED_PIPER_HAT = "pied_piper_hat"
    PIED_PIPER_PP = "pied_piper_pp"
    PIED_PIPER_SQUARE = "pied_piper_square"
    PIGGY_BANK = "piggy_bank"
    PILLS = "pills"
    PINTEREST = "pinterest"
    PINTEREST_P = "pinterest_p"
    PINTEREST_SQUARE = "pinterest_square"
    PIZZA_SLICE = "pizza_slice"
    PLACE_OF_WORSHIP = "place_of_worship"
    PLANE = "plane"
    PLANE_ARRIVAL = "plane_arrival"
    PLANE_DEPARTURE = "plane_departure"
    PLANE_SLASH = "plane_slash"
    PLAY = "play"
    PLAY_CIRCLE = "play_circle"
    PLAYSTATION = "playstation"
    PLUG = "plug"
    PLUS = "plus"
    PLUS_CIRCLE = "plus_circle"
    PLUS_SQUARE = "plus_square"
    PODCAST = "podcast"
    POLL = "poll"
    POLL_H = "poll_h"
    POO = "poo"
    POO_STORM = "poo_storm"
    POOP = "poop"
    PORTRAIT = "portrait"
    POUND_SIGN = "pound_sign"
    POWER_OFF = "power_off"
    PRAY = "pray"
    PRAYING_HANDS = "praying_hands"
    PRESCRIPTION = "prescription"
    PRESCRIPTION_BOTTLE = "prescription_bottle"
    PRESCRIPTION_BOTTLE_ALT = "prescription_bottle_alt"
    PRINT = "print"
    PROCEDURES = "procedures"
    PRODUCT_HUNT = "product_hunt"
    PROJECT_DIAGRAM = "project_diagram"
    PUMP_MEDICAL = "pump_medical"
    PUMP_SOAP = "pump_soap"
    PUSHED = "pushed"
    PUZZLE_PIECE = "puzzle_piece"
    PYTHON = "python"
    QQ = "qq"
    QRCODE = "qrcode"
    QUESTION = "question"
    QUESTION_CIRCLE = "question_circle"
    QUIDDITCH = "quidditch"
    QUINSCAPE = "quinscape"
    QUORA = "quora"
    QUOTE_LEFT = "quote_left"
    QUOTE_RIGHT = "quote_right"
    QURAN = "quran"
    R_PROJECT = "r_project"
    RADIATION = "radiation"
    RADIATION_ALT = "radiation_alt"
    RAINBOW = "rainbow"
    RANDOM = "random"
    RASPBERRY_PI = "raspberry_pi"
    RAVELRY = "ravelry"
    REACT = "react"
    REACTEUROPE = "reacteurope"
    README = "readme"
    REBEL = "rebel"
    RECEIPT = "receipt"
    RECORD_VINYL = "record_vinyl"
    RECYCLE = "recycle"
    RED_RIVER = "red_river"
    REDDIT = "reddit"
    REDDIT_ALIEN = "reddit_alien"
    REDDIT_SQUARE = "reddit_square"
    REDHAT = "redhat"
    REDO = "redo"
    REDO_ALT = "redo_alt"
    REGISTERED = "registered"
    REMOVE_FORMAT = "remove_format"
    RENREN = "renren"
    REPLY = "reply"
    REPLY_ALL = "reply_all"
    REPLYD = "replyd"
    REPUBLICAN = "republican"
    RESEARCHGATE = "researchgate"
    RESOLVING = "resolving"
    RESTROOM = "restroom"
    RETWEET = "retweet"
    REV = "rev"
    RIBBON = "ribbon"
    RING = "ring"
    ROAD = "road"
    ROBOT = "robot"
    ROCKET = "rocket"
    ROCKETCHAT = "rocketchat"
    ROCKRMS = "rockrms"
    ROUTE = "route"
    RSS = "rss"
    RSS_SQUARE = "rss_square"
    RUBLE_SIGN = "ruble_sign"
    RULER = "ruler"
    RULER_COMBINED = "ruler_combined"
    RULER_HORIZONTAL = "ruler_horizontal"
    RULER_VERTICAL = "ruler_vertical"
    RUNNING = "running"
    RUPEE_SIGN = "rupee_sign"
    RUST = "rust"
    SAD_CRY = "sad_cry"
    SAD_TEAR = "sad_tear"
    SAFARI = "safari"
    SALESFORCE = "salesforce"
    SASS = "sass"
    SATELLITE = "satellite"
    SATELLITE_DISH = "satellite_dish"
    SAVE = "save"
    SCHLIX = "schlix"
    SCHOOL = "school"
    SCREWDRIVER = "screwdriver"
    SCRIBD = "scribd"
    SCROLL = "scroll"
    SD_CARD = "sd_card"
    SEARCH = "search"
    SEARCH_DOLLAR = "search_dollar"
    SEARCH_LOCATION = "search_location"
    SEARCH_MINUS = "search_minus"
    SEARCH_PLUS = "search_plus"
    SEARCHENGIN = "searchengin"
    SEEDLING = "seedling"
    SELLCAST = "sellcast"
    SELLSY = "sellsy"
    SERVER = "server"
    SERVICESTACK = "servicestack"
    SHAPES = "shapes"
    SHARE = "share"
    SHARE_ALT = "share_alt"
    SHARE_ALT_SQUARE = "share_alt_square"
    SHARE_SQUARE = "share_square"
    SHEKEL_SIGN = "shekel_sign"
    SHIELD_ALT = "shield_alt"
    SHIELD_VIRUS = "shield_virus"
    SHIP = "ship"
    SHIPPING_FAST = "shipping_fast"
    SHIRTSINBULK = "shirtsinbulk"
    SHOE_PRINTS = "shoe_prints"
    SHOPIFY = "shopify"
    SHOPPING_BAG = "shopping_bag"
    SHOPPING_BASKET = "shopping_basket"
    SHOPPING_CART = "shopping_cart"
    SHOPWARE = "shopware"
    SHOWER = "shower"
    SHUTTLE_VAN = "shuttle_van"
    SIGN = "sign"
    SIGN_IN_ALT = "sign_in_alt"
    SIGN_LANGUAGE = "sign_language"
    SIGN_OUT_ALT = "sign_out_alt"
    SIGNAL = "signal"
    SIGNATURE = "signature"
    SIM_CARD = "sim_card"
    SIMPLYBUILT = "simplybuilt"
    SINK = "sink"
    SISTRIX = "sistrix"
    SITEMAP = "sitemap"
    SITH = "sith"
    SKATING = "skating"
    SKETCH = "sketch"
    SKIING = "skiing"
    SKIING_NORDIC = "skiing_nordic"
    SKULL = "skull"
    SKULL_CROSSBONES = "skull_crossbones"
    SKYATLAS = "skyatlas"
    SKYPE = "skype"
    SLACK = "slack"
    SLACK_HASH = "slack_hash"
    SLASH = "slash"
    SLEIGH = "sleigh"
    SLIDERS_H = "sliders_h"
    SLIDESHARE = "slideshare"
    SMILE = "smile"
    SMILE_BEAM = "smile_beam"
    SMILE_WINK = "smile_wink"
    SMOG = "smog"
    SMOKING = "smoking"
    SMOKING_BAN = "smoking_ban"
    SMS = "sms"
    SNAPCHAT = "snapchat"
    SNAPCHAT_GHOST = "snapchat_ghost"
    SNAPCHAT_SQUARE = "snapchat_square"
    SNOWBOARDING = "snowboarding"
    SNOWFLAKE = "snowflake"
    SNOWMAN = "snowman"
    SNOWPLOW = "snowplow"
    SOAP = "soap"
    SOCKS = "socks"
    SOLAR_PANEL = "solar_panel"
    SORT = "sort"
    SORT_ALPHA_DOWN = "sort_alpha_down"
    SORT_ALPHA_DOWN_ALT = "sort_alpha_down_alt"
    SORT_ALPHA_UP = "sort_alpha_up"
    SORT_ALPHA_UP_ALT = "sort_alpha_up_alt"
    SORT_AMOUNT_DOWN = "sort_amount_down"
    SORT_AMOUNT_DOWN_ALT = "sort_amount_down_alt"
    SORT_AMOUNT_UP = "sort_amount_up"
    SORT_AMOUNT_UP_ALT = "sort_amount_up_alt"
    SORT_DOWN = "sort_down"
    SORT_NUMERIC_DOWN = "sort_numeric_down"
    SORT_NUMERIC_DOWN_ALT = "sort_numeric_down_alt"
    SORT_NUMERIC_UP = "sort_numeric_up"
    SORT_NUMERIC_UP_ALT = "sort_numeric_up_alt"
    SORT_UP = "sort_up"
    SOUNDCLOUD = "soundcloud"
    SOURCETREE = "sourcetree"
    SPA = "spa"
    SPACE_SHUTTLE = "space_shuttle"
    SPEAKAP = "speakap"
    SPEAKER_DECK = "speaker_deck"
    SPELL_CHECK = "spell_check"
    SPIDER = "spider"
    SPINNER = "spinner"
    SPLOTCH = "splotch"
    SPOTIFY = "spotify"
    SPRAY_CAN = "spray_can"
    SQUARE = "square"
    SQUARE_FULL = "square_full"
    SQUARE_ROOT_ALT = "square_root_alt"
    SQUARESPACE = "squarespace"
    STACK_EXCHANGE = "stack_exchange"
    STACK_OVERFLOW = "stack_overflow"
    STACKPATH = "stackpath"
    STAMP = "stamp"
    STAR = "star"
    STAR_AND_CRESCENT = "star_and_crescent"
    STAR_HALF = "star_half"
    STAR_HALF_ALT = "star_half_alt"
    STAR_OF_DAVID = "star_of_david"
    STAR_OF_LIFE = "star_of_life"
    STAYLINKED = "staylinked"
    STEAM = "steam"
    STEAM_SQUARE = "steam_square"
    STEAM_SYMBOL = "steam_symbol"
    STEP_BACKWARD = "step_backward"
    STEP_FORWARD = "step_forward"
    STETHOSCOPE = "stethoscope"
    STICKER_MULE = "sticker_mule"
    STICKY_NOTE = "sticky_note"
    STOP = "stop"
    STOP_CIRCLE = "stop_circle"
    STOPWATCH = "stopwatch"
    STOPWATCH_20 = "stopwatch_20"
    STORE = "store"
    STORE_ALT = "store_alt"
    STORE_ALT_SLASH = "store_alt_slash"
    STORE_SLASH = "store_slash"
    STRAVA = "strava"
    STREAM = "stream"
    STREET_VIEW = "street_view"
    STRIKETHROUGH = "strikethrough"
    STRIPE = "stripe"
    STRIPE_S = "stripe_s"
    STROOPWAFEL = "stroopwafel"
    STUDIOVINARI = "studiovinari"
    STUMBLEUPON = "stumbleupon"
    STUMBLEUPON_CIRCLE = "stumbleupon_circle"
    SUBSCRIPT = "subscript"
    SUBWAY = "subway"
    SUITCASE = "suitcase"
    SUITCASE_ROLLING = "suitcase_rolling"
    SUN = "sun"
    SUPERPOWERS = "superpowers"
    SUPERSCRIPT = "superscript"
    SUPPLE = "supple"
    SURPRISE = "surprise"
    SUSE = "suse"
    SWATCHBOOK = "swatchbook"
    SWIFT = "swift"
    SWIMMER = "swimmer"
    SWIMMING_POOL = "swimming_pool"
    SYMFONY = "symfony"
    SYNAGOGUE = "synagogue"
    SYNC = "sync"
    SYNC_ALT = "sync_alt"
    SYRINGE = "syringe"
    TABLE = "table"
    TABLE_TENNIS = "table_tennis"
    TABLET = "tablet"
    TABLET_ALT = "tablet_alt"
    TABLETS = "tablets"
    TACHOMETER_ALT = "tachometer_alt"
    TAG = "tag"
    TAGS = "tags"
    TAPE = "tape"
    TASKS = "tasks"
    TAXI = "taxi"
    TEAMSPEAK = "teamspeak"
    TEETH = "teeth"
    TEETH_OPEN = "teeth_open"
    TELEGRAM = "telegram"
    TELEGRAM_PLANE = "telegram_plane"
    TEMPERATURE_HIGH = "temperature_high"
    TEMPERATURE_LOW = "temperature_low"
    TENCENT_WEIBO = "tencent_weibo"
    TENGE = "tenge"
    TERMINAL = "terminal"
    TEXT_HEIGHT = "text_height"
    TEXT_WIDTH = "text_width"
    TH = "th"
    TH_LARGE = "th_large"
    TH_LIST = "th_list"
    THE_RED_YETI = "the_red_yeti"
    THEATER_MASKS = "theater_masks"
    THEMECO = "themeco"
    THEMEISLE = "themeisle"
    THERMOMETER = "thermometer"
    THERMOMETER_EMPTY = "thermometer_empty"
    THERMOMETER_FULL = "thermometer_full"
    THERMOMETER_HALF = "thermometer_half"
    THERMOMETER_QUARTER = "thermometer_quarter"
    THERMOMETER_THREE_QUARTERS = "thermometer_three_quarters"
    THINK_PEAKS = "think_peaks"
    THUMBS_DOWN = "thumbs_down"
    THUMBS_UP = "thumbs_up"
    THUMBTACK = "thumbtack"
    TICKET_ALT = "ticket_alt"
    TIKTOK = "tiktok"
    TIMES = "times"
    TIMES_CIRCLE = "times_circle"
    TINT = "tint"
    TINT_SLASH = "tint_slash"
    TIRED = "tired"
    TOGGLE_OFF = "toggle_off"
    TOGGLE_ON = "toggle_on"
    TOILET = "toilet"
    TOILET_PAPER = "toilet_paper"
    TOILET_PAPER_SLASH = "toilet_paper_slash"
    TOOLBOX = "toolbox"
    TOOLS = "tools"
    TOOTH = "tooth"
    TORAH = "torah"
    TORII_GATE = "torii_gate"
    TRACTOR = "tractor"
    TRADE_FEDERATION = "trade_federation"
    TRADEMARK = "trademark"
    TRAFFIC_LIGHT = "traffic_light"
    TRAILER = "trailer"
    TRAIN = "train"
    TRAM = "tram"
    TRANSGENDER = "transgender"
    TRANSGENDER_ALT = "transgender_alt"
    TRASH = "trash"
    TRASH_ALT = "trash_alt"
    TRASH_RESTORE = "trash_restore"
    TRASH_RESTORE_ALT = "trash_restore_alt"
    TREE = "tree"
    TRELLO = "trello"
    TRIPADVISOR = "tripadvisor"
    TROPHY = "trophy"
    TRUCK = "truck"
    TRUCK_LOADING = "truck_loading"
    TRUCK_MONSTER = "truck_monster"
    TRUCK_MOVING = "truck_moving"
    TRUCK_PICKUP = "truck_pickup"
    TSHIRT = "tshirt"
    TTY = "tty"
    TUMBLR = "tumblr"
    TUMBLR_SQUARE = "tumblr_square"
    TV = "tv"
    TWITCH = "twitch"
    TWITTER = "twitter"
    TWITTER_SQUARE = "twitter_square"
    TYPO3 = "typo3"
    UBER = "uber"
    UBUNTU = "ubuntu"
    UIKIT = "uikit"
    UMBRACO = "umbraco"
    UMBRELLA = "umbrella"
    UMBRELLA_BEACH = "umbrella_beach"
    UNCHARTED = "uncharted"
    UNDERLINE = "underline"
    UNDO = "undo"
    UNDO_ALT = "undo_alt"
    UNIREGISTRY = "uniregistry"
    UNITY = "unity"
    UNIVERSAL_ACCESS = "universal_access"
    UNIVERSITY = "university"
    UNLINK = "unlink"
    UNLOCK = "unlock"
    UNLOCK_ALT = "unlock_alt"
    UNSPLASH = "unsplash"
    UNTAPPD = "untappd"
    UPLOAD = "upload"
    UPS = "ups"
    USB = "usb"
    USER = "user"
    USER_ALT = "user_alt"
    USER_ALT_SLASH = "user_alt_slash"
    USER_ASTRONAUT = "user_astronaut"
    USER_CHECK = "user_check"
    USER_CIRCLE = "user_circle"
    USER_CLOCK = "user_clock"
    USER_COG = "user_cog"
    USER_EDIT = "user_edit"
    USER_FRIENDS = "user_friends"
    USER_GRADUATE = "user_graduate"
    USER_INJURED = "user_injured"
    USER_LOCK = "user_lock"
    USER_MD = "user_md"
    USER_MINUS = "user_minus"
    USER_NINJA = "user_ninja"
    USER_NURSE = "user_nurse"
    USER_PLUS = "user_plus"
    USER_SECRET = "user_secret"
    USER_SHIELD = "user_shield"
    USER_SLASH = "user_slash"
    USER_TAG = "user_tag"
    USER_TIE = "user_tie"
    USER_TIMES = "user_times"
    USERS = "users"
    USERS_COG = "users_cog"
    USERS_SLASH = "users_slash"
    USPS = "usps"
    USSUNNAH = "ussunnah"
    UTENSIL_SPOON = "utensil_spoon"
    UTENSILS = "utensils"
    VAADIN = "vaadin"
    VECTOR_SQUARE = "vector_square"
    VENUS = "venus"
    VENUS_DOUBLE = "venus_double"
    VENUS_MARS = "venus_mars"
    VEST = "vest"
    VEST_PATCHES = "vest_patches"
    VIACOIN = "viacoin"
    VIADEO = "viadeo"
    VIADEO_SQUARE = "viadeo_square"
    VIAL = "vial"
    VIALS = "vials"
    VIBER = "viber"
    VIDEO = "video"
    VIDEO_SLASH = "video_slash"
    VIHARA = "vihara"
    VIMEO = "vimeo"
    VIMEO_SQUARE = "vimeo_square"
    VIMEO_V = "vimeo_v"
    VINE = "vine"
    VIRUS = "virus"
    VIRUS_SLASH = "virus_slash"
    VIRUSES = "viruses"
    VK = "vk"
    VNV = "vnv"
    VOICEMAIL = "voicemail"
    VOLLEYBALL_BALL = "volleyball_ball"
    VOLUME_DOWN = "volume_down"
    VOLUME_MUTE = "volume_mute"
    VOLUME_OFF = "volume_off"
    VOLUME_UP = "volume_up"
    VOTE_YEA = "vote_yea"
    VR_CARDBOARD = "vr_cardboard"
    VUEJS = "vuejs"
    WALKING = "walking"
    WALLET = "wallet"
    WAREHOUSE = "warehouse"
    WATCHMAN_MONITORING = "watchman_monitoring"
    WATER = "water"
    WAVE_SQUARE = "wave_square"
    WAZE = "waze"
    WEEBLY = "weebly"
    WEIBO = "weibo"
    WEIGHT = "weight"
    WEIGHT_HANGING = "weight_hanging"
    WEIXIN = "weixin"
    WHATSAPP = "whatsapp"
    WHATSAPP_SQUARE = "whatsapp_square"
    WHEELCHAIR = "wheelchair"
    WHMCS = "whmcs"
    WIFI = "wifi"
    WIKIPEDIA_W = "wikipedia_w"
    WIND = "wind"
    WINDOW_CLOSE = "window_close"
    WINDOW_MAXIMIZE = "window_maximize"
    WINDOW_MINIMIZE = "window_minimize"
    WINDOW_RESTORE = "window_restore"
    WINDOWS = "windows"
    WINE_BOTTLE = "wine_bottle"
    WINE_GLASS = "wine_glass"
    WINE_GLASS_ALT = "wine_glass_alt"
    WIX = "wix"
    WIZARDS_OF_THE_COAST = "wizards_of_the_coast"
    WODU = "wodu"
    WOLF_PACK_BATTALION = "wolf_pack_battalion"
    WON_SIGN = "won_sign"
    WORDPRESS = "wordpress"
    WORDPRESS_SIMPLE = "wordpress_simple"
    WPBEGINNER = "wpbeginner"
    WPEXPLORER = "wpexplorer"
    WPFORMS = "wpforms"
    WPRESSR = "wpressr"
    WRENCH = "wrench"
    X_RAY = "x_ray"
    XBOX = "xbox"
    XING = "xing"
    XING_SQUARE = "xing_square"
    Y_COMBINATOR = "y_combinator"
    YAHOO = "yahoo"
    YAMMER = "yammer"
    YANDEX = "yandex"
    YANDEX_INTERNATIONAL = "yandex_international"
    YARN = "yarn"
    YELP = "yelp"
    YEN_SIGN = "yen_sign"
    YIN_YANG = "yin_yang"
    YOAST = "yoast"
    YOUTUBE = "youtube"
    YOUTUBE_SQUARE = "youtube_square"
    ZHIHU = "zhihu"



"""Базовые классы."""

from enum import Enum


class BaseTag:
    """Базовый класс для тегов."""

    def __init__(
        self: "BaseTag",
        tag_stereo: str,
    ) -> None:
        """Create base tag."""
        self.__tagStereo = tag_stereo

    @staticmethod
    def _repr_if_not_none(
        value: str | None,
        text: str,
    ) -> str:
        if value is None:
            return ""
        return f', ${text}="{value}"'

    def repr_declaration(self: "BaseTag") -> str:
        """Объявление тега в начале файла."""
        raise NotImplementedError("Метод не переопределен.")

    def repr_func_param(self: "BaseTag") -> str:
        """Представление тега для параметра функции."""
        return f'$tags="{self.__tagStereo}"'

    def repr_tag(self: "BaseTag") -> str:
        """Только имя тега."""
        return f"{self.__tagStereo}"


class BaseSprites(Enum):
    """Базовая библиотека."""


class BaseSprite:
    """Базовый класс для изображений."""

    def __init__(
        self: "BaseSprite",
        common: str,
        common_sprite: str,
        sprite: BaseSprites,
    ) -> None:
        """Create base sprite."""
        self.__common = common
        self.__common_sprite = common_sprite
        self.__sprite = sprite

    @property
    def common(self: "BaseSprite") -> str:
        """Общая библиотека для импорта."""
        return f"!include <{self.__common}>"

    @property
    def sprite_full(self: "BaseSprite") -> str:
        """Импорт спрайта."""
        return f"!include <{self.__common_sprite}/{self.__sprite.value}>"

    @property
    def sprite_short(self: "BaseSprite") -> str:
        """Импорт спрайта."""
        return self.__sprite.value


class BaseC4Element:
    """Базовый элемент диаграмм."""

    def __init__(
        self: "BaseC4Element",
        label: str,
        sprite: BaseSprite | None,
        link: str | None,
        tags: tuple[BaseTag] | None,
    ) -> None:
        """Create BaseC4Element."""
        self.__alias = str(id(self)).replace("-", "_")
        self.__label = label
        self.__sprite = sprite
        self.__link = link
        self.__tags = tags

    @property
    def all_sprites(self: "BaseC4Element") -> list[BaseSprite]:
        """Возвращает все спрайты."""
        sprites: list[BaseSprite] = []
        if self.__sprite is not None:
            sprites.append(self.__sprite or None)
        return sprites

    @property
    def all_tags(self: "BaseC4Element") -> list[BaseTag]:
        """Возвращает список всех тегов."""
        tags: list[BaseTag] = []
        tags.extend(self.__tags or [])
        return tags

    @property
    def format_alias(self: "BaseC4Element") -> str:
        """Возвращает alias."""
        return self.__alias

    @property
    def _format_label(self: "BaseC4Element") -> str:
        """Представление для label."""
        return self._repr_if_not_none("label", self.__label)

    @property
    def _format_link(self: "BaseC4Element") -> str:
        """Ссылка."""
        return self._repr_if_not_none("link", self.__link)

    @property
    def _format_sprite(self: "BaseC4Element") -> str:
        """Представление для спрайтов."""
        if self.__sprite is None:
            return ""
        # return self.__sprite.sprite_short
        return self._repr_if_not_none("sprite", self.__sprite.sprite_short)

    @property
    def _format_tags(self: "BaseC4Element") -> str:
        """Тег."""
        if self.__tags is None:
            return ""
        return f', $tags="{"+".join([t.repr_tag() for t in self.__tags])}"'

    @staticmethod
    def _repr_if_not_none(
        text: str,
        value: str | None,
    ) -> str:
        if value is None:
            return ""
        return f', ${text}="{value}"'


class BaseRelation:
    """Базовый класс для отношений."""

    def __init__(
        self: "BaseRelation",
        kind: str,
        links: tuple[BaseC4Element, BaseC4Element],
        label: str,
        techn: str,
        descr: str,
        link: str | None,
        tags: tuple[BaseTag, ...] | None,
    ) -> None:
        """Создает Relation."""
        self.__kind = kind
        self.__links = links
        self.__label = label
        self.__techn = techn
        self.__descr = descr
        self.__link = link
        self.__tags = tags

    @property
    def _repr_link(self: "BaseRelation") -> str:
        """Представление ссылки."""
        if self.__link is None:
            return ""
        return f', $link="{self.__link}"'

    @property
    def repr_tags(self: "BaseRelation") -> str:
        """Тег."""
        if self.__tags is None:
            return ""
        return f', $tags="{"+".join([t.repr_tag() for t in self.__tags])}"'

    @property
    def all_tags(self: "BaseRelation") -> list[BaseTag]:
        """Возвращает список всех тегов."""
        tags: list[BaseTag] = []
        if self.__tags is not None:
            tags.extend(self.__tags)
        return tags

    def __repr__(self: "BaseRelation") -> str:
        """Return string representation."""
        template = """
{kind}({link_from}, {link_to}, "{label}", "{techn}", "{descr}"{link}{tags})"""
        return template.format(
            kind=self.__kind,
            link_from=self.__links[0].format_alias,
            link_to=self.__links[1].format_alias,
            label=self.__label,
            techn=self.__techn,
            descr=self.__descr,
            link=self._repr_link,
            tags=self.repr_tags,
        )



"""Представление для https://github.com/blockdiag/nwdiag.

Примеры диаграмм - http://blockdiag.com/en/nwdiag/nwdiag-examples.html
"""

from typing import NamedTuple


from konstantin_docs.dia.base import BaseDiagram, Image
from konstantin_docs.service.kroki import (
    DiagramTypes,
    OutputFormats,
    get_image,
)


class NodeInNetwork(NamedTuple):
    """Узел в сети."""

    node: "Node"
    address: list[str]


class Node:
    """Узел."""

    def __init__(self: "Node", name: str) -> None:
        """Construct node."""
        self.__name = name

    @property
    def name(self: "Node") -> str:
        """Node name."""
        return self.__name


class Network:
    """Сеть."""

    def __init__(
        self: "Network",
        name: str = "",
        address: str = None,
    ) -> None:
        """Construct network."""
        self.__name = name
        self.__address = address
        self.__nodes: list[NodeInNetwork] = []

    def __repr__(self: "Network") -> str:
        """Represent string."""
        out = ""
        out += f"\tnetwork {self.__name} {{\n"
        if self.__address is not None:
            out += f'\t\taddress = "{self.__address}"\n'
        for node in self.__nodes:
            address = ", ".join(node.address)
            out += f'\t\t{node.node.name} [address = "{address}"]\n'
        out += "\t}\n"
        return out

    def add_node(self: "Network", node: Node, address: set[str] = ()) -> None:
        """Добавить узел к сети."""
        self.__nodes.append(NodeInNetwork(node=node, address=address))


class PeerNetwork:
    """Одноранговая сеть."""

    def __init__(
        self: "PeerNetwork",
        node1: NodeInNetwork,
        node2: NodeInNetwork,
    ) -> None:
        """Создать однорангувую сеть."""
        self.__node1 = node1
        self.__node2 = node2

    def __repr__(self: "PeerNetwork") -> str:
        """Represent string."""
        out = ""
        out += f"\t{self.__node1.name}\n"
        out += f"\t{self.__node2.name}\n"
        out += f"\t{self.__node1.name} -- {self.__node2.name}\n"
        return out


class Group:
    """Группа узлов."""

    def __init__(
        self: "Group",
        name: str,
        nodes: set[Node],
        color: str | None = None,
    ) -> None:
        """Construct group."""
        self.__name = name
        self.__color = color
        self.__nodes = nodes

    def __repr__(self: "Group") -> str:
        """Represent string."""
        out = ""
        out += f"\tgroup {self.__name} {{\n"
        if self.__color is not None:
            out += f'\t\tcolor = "{self.__color}";\n'
        for node in self.__nodes:
            out += f"\t\t{node.name};\n"
        out += "\t}\n"
        return out


class Diagram(BaseDiagram):
    """Общая диаграмма."""

    def __init__(
        self: "Diagram",
        filename: str,
        networks: set[Network],
        peer_networks: set[PeerNetwork] = (),
        groups: set[Group] = (),
    ) -> None:
        """Construct diagram."""
        super().__init__(filename)
        self.__networks = networks
        self.__peer_networks = peer_networks
        self.__groups = groups

    def add_network(self: "Diagram", network: Network) -> None:
        """Добавить сеть на диаграмму."""
        self.__networks.append(network)

    def get_images(self: "Diagram") -> tuple[Image]:
        """Возвращает кортеж изображений."""
        images: list[Image] = []
        for fmt in (OutputFormats.PNG, OutputFormats.SVG):
            images.append(
                Image(
                    filename=self.filename + "." + fmt.value,
                    content=get_image(
                        source=repr(self),
                        diagram_type=DiagramTypes.NWDIAG,
                        output_format=fmt,
                    ),
                ),
            )
        return tuple(images)

    def __repr__(self: "Diagram") -> str:
        """Represent string."""
        out = ""
        out += "nwdiag {\n"
        for net in self.__peer_networks:
            out += repr(net)
        for net in self.__networks:
            out += repr(net)
        for group in self.__groups:
            out += repr(group)
        out += "}\n"
        return out



"""Уровень 1 - context."""

from .base import BaseC4Element as _BaseC4Element
from .base import BaseSprite
from .base import BaseTag
from .container import BaseContainer as _BaseContainer
from .tag import ElementTag as _ElementTag


class BaseContext(_BaseC4Element):
    """BaseContext."""

    __descr: str | None
    __boundary_type: str | None
    __links_container: list[_BaseContainer] | None

    def __init__(
        self: "BaseContext",
        label: str,
        descr: str | None,
        sprite: BaseSprite | None,
        link: str | None,
        tags: tuple[_ElementTag] | None,
        boundary_type: str | None,
        links_container: list[_BaseContainer] | None,
    ) -> None:
        """Создать BaseContext."""
        super().__init__(
            label=label,
            sprite=sprite,
            link=link,
            tags=tags,
        )
        self.__descr = descr
        self.__boundary_type = boundary_type
        self.__links_container = links_container

    @property
    def _format_links_container(self: "BaseContext") -> str:
        """Возвращает список вложенных контейнеров в виде строки."""
        if self.__links_container is None:
            return ""
        links_container_str: str = "".join(
            [repr(container) for container in self.__links_container],
        )
        return f"{{\n\t{links_container_str}\n}}"

    @property
    def _repr_inside_pths(self: "BaseContext") -> str:
        """Возвращает содержимое внутри скобок + вложенные контейнеры."""
        return (
            "({alias}{label}{descr}{sprite}{tags}{link}{type})"
            "{links_container}\n"
        ).format(
            alias=self.format_alias,
            label=self._format_label,
            descr=self._repr_if_not_none("descr", self.__descr),
            sprite=self._format_sprite,
            tags=self._format_tags,
            link=self._format_link,
            type=self._repr_if_not_none("type", self.__boundary_type),
            links_container=self._format_links_container,
        )

    @_BaseC4Element.all_sprites.getter
    def all_sprites(self: "BaseContext") -> list[BaseSprite]:
        """Возвращает все спрайты."""
        sprites: list[BaseSprite] = super().all_sprites
        sprites.extend(
            [
                all_sprites
                for link in self.__links_container or []
                for all_sprites in link.all_sprites
            ],
        )
        return sprites

    @_BaseC4Element.all_tags.getter
    def all_tags(self: "BaseContext") -> list[BaseTag]:
        """Возвращает все теги.

        :return: список тегов
        """
        tags: list[BaseTag] = super().all_tags
        tags.extend(
            [
                tags
                for link in self.__links_container or []
                for tags in link.all_tags
            ],
        )
        return tags

    def __repr__(self: "BaseContext") -> str:
        """Return string representation.

        :raises NotImplementedError: метод не переопределен
        """
        raise NotImplementedError("Метод не переопределен")


class Person(BaseContext):
    """Person."""

    def __init__(
        self: "Person",
        label: str,
        descr: str = "",
        sprite: BaseSprite | None = None,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать Person."""
        super().__init__(
            label=label,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "Person") -> str:
        """Return string representation."""
        return f"Person{self._repr_inside_pths}"


class PersonExt(BaseContext):
    """PersonExt."""

    def __init__(
        self: "PersonExt",
        label: str,
        descr: str = "",
        sprite: BaseSprite | None = None,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать PersonExt."""
        super().__init__(
            label=label,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "PersonExt") -> str:
        """Return string representation."""
        return f"Person_Ext{self._repr_inside_pths}"


class System(BaseContext):
    """System."""

    def __init__(
        self: "System",
        label: str,
        descr: str = "",
        sprite: BaseSprite | None = None,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать System."""
        super().__init__(
            label=label,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "System") -> str:
        """Return string representation."""
        return f"System{self._repr_inside_pths}"


class SystemDb(BaseContext):
    """SystemDb."""

    def __init__(
        self: "SystemDb",
        label: str,
        descr: str = "",
        sprite: BaseSprite | None = None,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать System."""
        super().__init__(
            label=label,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "SystemDb") -> str:
        """Return string representation."""
        return f"SystemDb{self._repr_inside_pths}"


class SystemQueue(BaseContext):
    """SystemQueue."""

    def __init__(
        self: "SystemQueue",
        label: str,
        descr: str = "",
        sprite: BaseSprite | None = None,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать System."""
        super().__init__(
            label=label,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "SystemQueue") -> str:
        """Return string representation."""
        return f"SystemQueue{self._repr_inside_pths}"


class SystemExt(BaseContext):
    """SystemExt."""

    def __init__(
        self: "SystemExt",
        label: str,
        descr: str = "",
        sprite: BaseSprite | None = None,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать SystemExt."""
        super().__init__(
            label=label,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "SystemExt") -> str:
        """Return string representation."""
        return f"System_Ext{self._repr_inside_pths}"


class SystemDbExt(BaseContext):
    """SystemDbExt."""

    def __init__(
        self: "SystemDbExt",
        label: str,
        descr: str = "",
        sprite: BaseSprite | None = None,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать SystemDbExt."""
        super().__init__(
            label=label,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "SystemDbExt") -> str:
        """Return string representation."""
        return f"SystemDb_Ext{self._repr_inside_pths}"


class SystemQueueExt(BaseContext):
    """SystemQueueExt."""

    def __init__(
        self: "SystemQueueExt",
        label: str,
        descr: str = "",
        sprite: BaseSprite | None = None,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать SystemQueueExt."""
        super().__init__(
            label=label,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "SystemQueueExt") -> str:
        """Return string representation."""
        return f"SystemQueue_Ext{self._repr_inside_pths}"


class Boundary(BaseContext):
    """System."""

    __boundary_type: str

    def __init__(
        self: "Boundary",
        label: str,
        boundary_type: str = "",
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать System."""
        super().__init__(
            label=label,
            descr=None,
            sprite=None,
            link=link,
            tags=tags,
            boundary_type=boundary_type,
            links_container=links_container,
        )
        self.__boundary_type = boundary_type

    def __repr__(self: "Boundary") -> str:
        """Return string representation."""
        return f"Boundary{self._repr_inside_pths}"


class EnterpriseBoundary(BaseContext):
    """EnterpriseBoundary."""

    def __init__(
        self: "EnterpriseBoundary",
        label: str,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать EnterpriseBoundary."""
        super().__init__(
            label=label,
            descr=None,
            sprite=None,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "EnterpriseBoundary") -> str:
        """Return string representation."""
        return f"Enterprise_Boundary{self._repr_inside_pths}"


class SystemBoundary(BaseContext):
    """SystemBoundary."""

    def __init__(
        self: "SystemBoundary",
        label: str,
        link: str | None = None,
        tags: tuple[_ElementTag] | None = None,
        links_container: list[_BaseContainer] | None = None,
    ) -> None:
        """Создать SystemBoundary."""
        super().__init__(
            label=label,
            descr=None,
            sprite=None,
            link=link,
            tags=tags,
            boundary_type=None,
            links_container=links_container,
        )

    def __repr__(self: "SystemBoundary") -> str:
        """Return string representation."""
        return f"System_Boundary{self._repr_inside_pths}"



"""C4 with PlantUML."""

from . import component, container, context, rel, sprite, tag
from .c4 import C4

__all__: list[str] = [
    "C4",
    "component",
    "container",
    "context",
    "rel",
    "sprite",
    "tag",
]
